/// <reference types="jest" />
import { TsJestGlobalOptions } from './types';
import { ConfigSet } from './config/config-set';
import { Logger } from 'bs-logger';
/**
 * @internal
 */
export declare const INSPECT_CUSTOM: symbol;
export declare class TsJestTransformer implements jest.Transformer {
    private static _configSetsIndex;
    private static _lastTransformerId;
    static readonly lastTransformerId: number;
    private static readonly _nextTransformerId;
    readonly logger: Logger;
    readonly id: number;
    readonly options: TsJestGlobalOptions;
    constructor(baseOptions?: TsJestGlobalOptions);
    configsFor(jestConfig: jest.ProjectConfig | string): ConfigSet;
    process(input: string, filePath: jest.Path, jestConfig: jest.ProjectConfig, transformOptions?: jest.TransformOptions): jest.TransformedSource | string;
    /**
     * Jest uses this to cache the compiled version of a file
     *
     * @see https://github.com/facebook/jest/blob/v23.5.0/packages/jest-runtime/src/script_transformer.js#L61-L90
     * @param fileContent The content of the file
     * @param filePath The full path to the file
     * @param jestConfigStr The JSON-encoded version of jest config
     * @param transformOptions.instrument Whether the content will be instrumented by our transformer (always false)
     * @param transformOptions.rootDir Jest current rootDir
     */
    getCacheKey(fileContent: string, filePath: string, jestConfigStr: string, transformOptions?: {
        instrument?: boolean;
        rootDir?: string;
    }): string;
}
